package ast;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.List;

import types.Type;

public abstract class Node {
	public int line;
	public int col;

	public Node(int line, int col) {
		this.line = line;
		this.col = col;
	}

	public abstract String label();

	public void typeError(Type t1, Type t2) {
		throw new RuntimeException("tipos incompatíveis, linha " + line + ", coluna " + col + ": " + t1 + ", " + t2);
	}

	public void arityError() {
		throw new RuntimeException("aridade errada na chamada, linha " + line + ", coluna " + col);
	}

	public void printDot(String filename) {
		BufferedWriter buf;
		try {
			buf = new BufferedWriter(new FileWriter(filename));
			try {
				buf.write("digraph tree {\n");
				StringBuffer sb = new StringBuffer();
				dot(sb);
				buf.write(sb.toString());
				buf.write("}\n");
			} finally {
				buf.close();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void dot(StringBuffer buf) {
		Class<? extends Node> c = this.getClass();
		buf.append("\"" + this.toString() + "\"");
		buf.append(" [label=\"");
		buf.append(label());
		buf.append("\"];\n");
		for(Field f: c.getDeclaredFields()) {
			Object obj;
			try {
				obj = f.get(this);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			}
			if(obj instanceof PrimFunc) {
				PrimFunc pf = (PrimFunc)obj;
				if(pf != null) {
					buf.append("\"" + pf.toString() + "\"");
					buf.append(" [label=\"");
					buf.append(pf.label());
					buf.append("\"];\n");
					buf.append("\"" + this.toString() + "\"");
					buf.append(" -> ");
					buf.append("\"" + pf.toString() + "\"");
					buf.append(";\n");
				}
			} else if(obj instanceof Node) {
				Node child = (Node)obj;
				if(child != null) {
					child.dot(buf);
					buf.append("\"" + this.toString() + "\"");
					buf.append(" -> ");
					buf.append("\"" + child.toString() + "\"");
					buf.append(";\n");
				}
			} else if(obj instanceof List) {
				List l = (List)obj;
				for(Object o: l) {
					Node child = (Node)o;
					child.dot(buf);
					buf.append("\"" + this.toString() + "\"");
					buf.append(" -> ");
					buf.append("\"" + child.toString() + "\"");
					buf.append(";\n");
				}
			}
		}
	}

}


